@@ -7,7 +7,7 @@ use crate::{SubscriptionId, Transaction, ID};
 use lib0::any::Any;
 use std::cell::UnsafeCell;
 use std::collections::{HashSet, VecDeque};
-use std::ops::Deref;
+use std::ops::{Deref, DerefMut};
 
 /// A collection used to store data in an indexed sequence structure. This type is internally
 /// implemented as a double linked list, which may squash values inserted directly one after another
@@ -157,6 +157,12 @@ impl AsRef<Branch> for Array {
     }
 }
 
+impl AsMut<Branch> for Array {
+    fn as_mut(&mut self) -> &mut Branch {
+        self.0.deref_mut()
+    }
+}
+
 pub struct ArrayIter<'a> {
     content: VecDeque<Value>,
     ptr: Option<&'a BlockPtr>,
@@ -254,8 +260,8 @@ where
 
 /// Event generated by [Array::observe] method. Emitted during transaction commit phase.
 pub struct ArrayEvent {
+    pub(crate) current_target: BranchPtr,
     target: Array,
-    current_target: BranchPtr,
     change_set: UnsafeCell<Option<Box<ChangeSet<Change>>>>,
 }
 
@@ -307,8 +313,8 @@ impl ArrayEvent {
 mod test {
     use crate::test_utils::{exchange_updates, run_scenario, RngExt};
     use crate::types::map::PrelimMap;
-    use crate::types::{Change, Value};
-    use crate::{Doc, PrelimArray, Update, ID};
+    use crate::types::{Change, DeepObservable, Event, Path, PathSegment, Value};
+    use crate::{Doc, PrelimArray, StateVector, Update, ID};
     use lib0::any::Any;
     use rand::prelude::StdRng;
     use rand::Rng;
@@ -367,7 +373,7 @@ mod test {
         let a1 = t1.get_array("array");
 
         a1.insert(&mut t1, 0, "Hi");
-        let update = t1.encode_update_v1(); //d1.encode_state_as_update_v1
+        let update = d1.encode_state_as_update_v1(&StateVector::default());
 
         let mut t2 = d2.transact();
         t2.apply_update(Update::decode_v1(update.as_slice()));
@@ -956,4 +962,37 @@ mod test {
         let actual = t1.get_array("array").get(0);
         assert_eq!(actual, None);
     }
+
+    #[test]
+    fn observe_deep_event_order() {
+        let doc = Doc::with_client_id(1);
+        let mut array = {
+            let mut txn = doc.transact();
+            txn.get_array("array")
+        };
+
+        let paths = Rc::new(RefCell::new(Vec::new()));
+        let paths_copy = paths.clone();
+
+        let _sub = array.observe_deep(move |_txn, e| {
+            let path: Vec<Path> = e.iter().map(Event::path).collect();
+            paths_copy.borrow_mut().push(path);
+        });
+
+        array.insert(&mut doc.transact(), 0, PrelimMap::<String>::new());
+
+        {
+            let mut txn = doc.transact();
+            let map = array.get(0).unwrap().to_ymap().unwrap();
+            map.insert(&mut txn, "a", "a");
+            array.insert(&mut txn, 0, 0);
+        }
+
+        let expected = &[
+            vec![Path::default()],
+            vec![Path::default(), Path::from([PathSegment::Index(1)])],
+        ];
+        let actual = RefCell::borrow(&paths);
+        assert_eq!(actual.as_slice(), expected);
+    }
 }
