@@ -370,6 +370,7 @@ impl Text {
                             let content_len = item.content_len(encoding);
                             if len < content_len {
                                 let new_right = txn.store_mut().blocks.split_block(right, len);
+                                pos.left = Some(right);
                                 pos.right = new_right;
                                 break;
                             }
@@ -440,7 +441,6 @@ impl Text {
                 let mut item_ptr = BlockPtr::from(&mut item);
                 pos.right = Some(item_ptr);
                 item_ptr.integrate(txn, 0);
-
                 let local_block_list = txn
                     .store_mut()
                     .blocks
@@ -665,6 +665,12 @@ impl AsRef<Branch> for Text {
     }
 }
 
+impl AsMut<Branch> for Text {
+    fn as_mut(&mut self) -> &mut Branch {
+        self.0.deref_mut()
+    }
+}
+
 #[derive(Debug, PartialEq)]
 pub enum Diff {
     Insert(Value, Option<Box<Attrs>>),
@@ -672,8 +678,8 @@ pub enum Diff {
 
 /// Event generated by [Text::observe] method. Emitted during transaction commit phase.
 pub struct TextEvent {
+    pub(crate) current_target: BranchPtr,
     target: Text,
-    current_target: BranchPtr,
     delta: UnsafeCell<Option<Vec<Delta>>>,
 }
 
@@ -708,14 +714,14 @@ impl TextEvent {
     }
 
     pub(crate) fn get_delta(target: BranchPtr, txn: &Transaction) -> Vec<Delta> {
-        #[derive(Clone, Copy, Eq, PartialEq)]
+        #[derive(Debug, Clone, Copy, Eq, PartialEq)]
         enum Action {
             Insert,
             Retain,
             Delete,
         }
 
-        #[derive(Default)]
+        #[derive(Debug, Default)]
         struct DeltaAssembler {
             action: Option<Action>,
             insert: Option<Value>,
@@ -758,7 +764,7 @@ impl TextEvent {
                         } else {
                             Some(Box::new(self.attrs.clone()))
                         };
-                        self.delta.push(Delta::Retain(len, attrs))
+                        self.delta.push(Delta::Retain(len, attrs));
                     }
                 }
             }
@@ -1204,7 +1210,7 @@ mod test {
         txt1.insert(&mut t1, 0, "hello world");
         assert_eq!(txt1.to_string().as_str(), "hello world");
 
-        let u1 = t1.encode_update_v1(); // d1.encode_state_as_update_v1(&t1);
+        let u1 = d1.encode_state_as_update_v1(&StateVector::default());
 
         let d2 = Doc::with_client_id(2);
         let mut t2 = d2.transact();
@@ -1629,4 +1635,29 @@ mod test {
             assert_eq!(txt1.diff(&mut txn), expected);
         }
     }
+
+    #[test]
+    fn issue_101() {
+        let d1 = Doc::with_client_id(1);
+        let mut txt1 = d1.transact().get_text("text");
+        let delta = Rc::new(RefCell::new(None));
+        let delta_copy = delta.clone();
+
+        let attrs: Attrs = HashMap::from([("bold".into(), true.into())]);
+
+        txt1.insert(&mut d1.transact(), 0, "abcd");
+
+        let _sub = txt1.observe(move |txn, e| {
+            let mut d = delta_copy.borrow_mut();
+            *d = Some(e.delta(txn).to_vec());
+        });
+        txt1.format(&mut d1.transact(), 1, 2, attrs.clone());
+
+        let expected = vec![
+            Delta::Retain(1, None),
+            Delta::Retain(2, Some(Box::new(attrs))),
+        ];
+        let actual = delta.borrow();
+        assert_eq!(actual.as_ref(), Some(&expected));
+    }
 }
