@@ -300,19 +300,25 @@ impl Update {
     fn missing(block: &BlockCarrier, local_sv: &StateVector) -> Option<u64> {
         if let BlockCarrier::Block(block) = block {
             if let Block::Item(item) = block.as_ref() {
-                if let Some(origin) = item.origin {
-                    if origin.client != item.id.client && !local_sv.contains(&item.id) {
+                if let Some(origin) = &item.origin {
+                    if origin.client != item.id.client
+                        && origin.clock >= local_sv.get(&origin.client)
+                    {
                         return Some(origin.client);
                     }
-                } else if let Some(right_origin) = item.right_origin {
-                    if right_origin.client != item.id.client && !local_sv.contains(&item.id) {
+                } else if let Some(right_origin) = &item.right_origin {
+                    if right_origin.client != item.id.client
+                        && right_origin.clock >= local_sv.get(&right_origin.client)
+                    {
                         return Some(right_origin.client);
                     }
-                } else if let TypePtr::Branch(parent) = item.parent {
-                    if let Some(block) = parent.item {
-                        let parent_client = block.id().client;
-                        if parent_client != item.id.client && !local_sv.contains(&item.id) {
-                            return Some(parent_client);
+                } else if let TypePtr::Branch(parent) = &item.parent {
+                    if let Some(block) = &parent.item {
+                        let parent_id = block.id();
+                        if parent_id.client != item.id.client
+                            && parent_id.clock >= local_sv.get(&parent_id.client)
+                        {
+                            return Some(parent_id.client);
                         }
                     }
                 }
@@ -370,7 +376,7 @@ impl Update {
                 BlockCarrier::Skip(BlockRange { id, len })
             }
             BLOCK_GC_REF_NUMBER => {
-                let len: u32 = decoder.read_uvar();
+                let len: u32 = decoder.read_len();
                 Box::new(Block::GC(BlockRange { id, len })).into()
             }
             info => {
@@ -450,7 +456,7 @@ impl Update {
         encoder.write_uvar(sorted_clients.len());
         for (&client, (offset, blocks)) in sorted_clients {
             encoder.write_uvar(blocks.len());
-            encoder.write_uvar(client);
+            encoder.write_client(client);
 
             let mut block = blocks[0];
             encoder.write_uvar(block.id().clock + offset);
@@ -745,7 +751,8 @@ impl BlockCarrier {
     pub(crate) fn same_type(&self, other: &BlockCarrier) -> bool {
         match (self, other) {
             (BlockCarrier::Skip(_), BlockCarrier::Skip(_)) => true,
-            (a, b) => a.same_type(b),
+            (BlockCarrier::Block(a), BlockCarrier::Block(b)) => a.same_type(b),
+            (_, _) => false,
         }
     }
     pub(crate) fn id(&self) -> &ID {
